/**
 * @file
 * VuoList_VuoHidControl C type definition.
 *
 * @copyright Copyright © 2012–2017 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#pragma once

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoHidControl VuoList_VuoHidControl
 * A list of @ref VuoHidControl elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoHidControl_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoHidControl elements.
 */
typedef const struct VuoList_VuoHidControl_struct { void *l; } * VuoList_VuoHidControl;
/// @}
#endif

/**
 * Creates a new list of @ref VuoHidControl elements.
 */
VuoList_VuoHidControl VuoListCreate_VuoHidControl(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoHidControl to quickly initialize a large list.
 */
VuoList_VuoHidControl VuoListCreateWithCount_VuoHidControl(const unsigned long count, const VuoHidControl value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoHidControl VuoListCopy_VuoHidControl(const VuoList_VuoHidControl list);

/**
 * Returns the @ref VuoHidControl at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoHidControl VuoListGetValue_VuoHidControl(const VuoList_VuoHidControl list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoHidControl *VuoListGetData_VuoHidControl(const VuoList_VuoHidControl list);

/**
 * Changes the @ref VuoHidControl at @c index.
 * Index values start at 1.
 *
 * If `expandListIfNeeded` is false: If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 *
 * If `expandListIfNeeded` is true: If the list has less than `index` elements, it is expanded to accommodate the specified `index`.
 * If `index` is 0, the list is resized to contain 1 item set to `value`.
 */
void VuoListSetValue_VuoHidControl(const VuoList_VuoHidControl list, const VuoHidControl value, const unsigned long index, bool expandListIfNeeded);

/**
 * Inserts the `VuoHidControl` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoHidControl(const VuoList_VuoHidControl list, const VuoHidControl value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoHidControl(VuoList_VuoHidControl list, const VuoHidControl value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoHidControl(VuoList_VuoHidControl list, const VuoHidControl value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoHidControl(VuoList_VuoHidControl list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoHidControl_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoHidControl(VuoList_VuoHidControl list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoHidControl_areEqual(const VuoList_VuoHidControl a, const VuoList_VuoHidControl b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoHidControl_isLessThan(const VuoList_VuoHidControl a, const VuoList_VuoHidControl b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoHidControl(VuoList_VuoHidControl list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoHidControl(VuoList_VuoHidControl list);

/**
 * Returns a new list containing items from the original list in the range specified by `startIndex` and `itemCount`.
 *
 * Items in the new list are retained (not copied) from the original list.
 */
VuoList_VuoHidControl VuoListSubset_VuoHidControl(VuoList_VuoHidControl list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoHidControl(VuoList_VuoHidControl list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoHidControl(VuoList_VuoHidControl list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoHidControl(VuoList_VuoHidControl list);

/**
 * Removes the `VuoHidControl` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoHidControl(VuoList_VuoHidControl list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoHidControl(const VuoList_VuoHidControl list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoHidControl VuoList_VuoHidControl_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoHidControl_getJson(const VuoList_VuoHidControl value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoHidControl_getSummary(const VuoList_VuoHidControl value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoHidControl VuoList_VuoHidControl_makeFromString(const char *str);
char * VuoList_VuoHidControl_getString(const VuoList_VuoHidControl value);
/// @}

/**
 * @}
 */

#ifdef __cplusplus
}
#endif
